/*********************************************************/
/*  This source code is copyrighted by the author        */
/*  Please read COPYING  for the GNU licence policy.     */
/*********************************************************/
#include <vga.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include "tet.h"
#include "egafont.h"
static char *Font;
static int  FontHeight;

int egaShowText(char *text, int x, int y, int fgcol, int bgcol)
{
   unsigned char buff[160];		// Enough for 20 char-s
   unsigned char *buffer;
   unsigned char mask;
   int  buflen,i;
   unsigned char fbyte;
   unsigned char *to;
   unsigned char *textptr, textchar;
 
   buflen = strlen(text) << 3;
   // Attempting to save time for short texts
   
   if (buflen > 160)
     {  buffer = (char *) malloc(buflen);
        if (!buffer) return 4;
     } 
   else  buffer = buff;

   for (i=0; i<FontHeight; i++)
   { 
      textptr = text;
      to = buffer;
      while ((textchar = *textptr++) != '\0')
      {  fbyte = *(Font + textchar*FontHeight + i);
         mask = 0x80;
         while(mask)
         {  *to++ = (fbyte & mask) ? fgcol: bgcol;
            mask = mask >> 1;
         }
      }
    
      setrow(x, y+i, buflen, buffer);
   }
   
   if (buflen > 160) free (buffer);
   return 0;

}

int egaShowScaledText(char *text, int x, int y,
                      int fgcol, int bgcol, int hscale, int vscale)
{
   unsigned char *buffer;
   unsigned char mask;
   int  buflen,i, j;
   unsigned char fbyte, c;
   unsigned char *to;
   unsigned char *textptr,textchar;

   buflen = (strlen(text) << 3) * hscale;
   
   //  This is called rarely, so you don't have to worry about time 
   buffer = (char *) malloc(buflen);

   for (i=0; i<FontHeight; i++)
   {
      textptr = text;
      to = buffer;
      while ((textchar = *textptr++) != '\0')
      {  fbyte = *(Font + textchar*FontHeight + i);
         mask = 0x80;
 
         while(mask)
         {  c =  (fbyte & mask) ? fgcol: bgcol;
            j = hscale;
            while(--j >=0) *to++ = c;
            mask = mask >> 1;
         }
      }
      
      j = vscale;
      while (--j >= 0)
         setrow(x, y++, buflen, buffer);
   }
   free (buffer);
   return 0;

}

int egaSetFont(char *cFont)
{   int *iFont;
    if (!cFont) return 4;

    iFont = (int *)cFont;
    FontHeight = *iFont;
    
    Font = cFont + sizeof(int);
    return 0;
}
  
void egaFreeFont(char *cFont)
{
   if (!cFont) return;

   if (cFont + sizeof(int) == Font)
   {  Font = NULL; FontHeight = 0; }

   free(cFont);
}        


int egaGetFontHeight(void)
{   return FontHeight; }

int egaGetTextWidth(char *text)
{  return strlen(text) * 8; }

char *egaGetFontPtr(int chr)
{  return Font + (chr & 0xff) *FontHeight; }

char *egaLoadFont(char *fontFilename)
{
   int fd;
   int rc = 4; 
   size_t s;
   off_t  hdrlen;    // File header length

   char  *cFont = NULL;
   int   *iFont;
   int   fontheight;

   fd = open(fontFilename, O_RDONLY);
   if (fd < 0) return NULL;

   // Size of the font file
   s = (size_t) lseek(fd, 0, SEEK_END);

   fontheight = s>>8;       // Assume 256 characters, FontHeight bytes each
   hdrlen  = s - (fontheight << 8);
   
   if (hdrlen != 0 && hdrlen != 4) goto Ret;
   
   s -= hdrlen;
   lseek(fd, hdrlen, SEEK_SET);  // Bypassing header

   cFont = (char *) malloc(s + sizeof(int));
   if (!cFont) goto Ret;

   iFont = (int *)cFont;
   *iFont = fontheight;

   if (read(fd, cFont+sizeof(int), s) == s) rc = 0; 
 
Ret:  
   
   close(fd);
   if (rc)
   {  if (cFont) free(cFont);
      return NULL;
   }

   return cFont;
}


int egaShowCharSpecial(int b, int x, int y, int fgcol, int bgcol,
                       int hscale, int vscale, int flags)
{   char *buffer, *bufptr, c;
    unsigned char *fontptr, fbyte, mask, smask;
    int hflip;
    int i, y1, yincr, w;
    
    w = hscale << 3; 
    buffer = (char *) malloc(w);
    if (!buffer) return 4;
 
    hflip = flags & EGAFONT_HFLIP;
    if (flags & EGAFONT_VFLIP)
    { y += vscale * FontHeight - 1;
      yincr = -1;
    }
    else
      yincr = 1;
    
    smask = hflip? 1 : 0x80;
    
    b &= 0xff;
    fontptr = Font + b*FontHeight;
 
    y1 = FontHeight;
    while(--y1 >= 0)
    {  
        fbyte = *fontptr++;
        bufptr = buffer;
        mask = smask;
	while (mask)  
        {  c = fbyte & mask ? fgcol : bgcol;
           for ( i=0; i<hscale; i++) *bufptr++ = c;
           if (hflip) mask <<=1; else mask >>= 1; 
        }   

        for (i=0; i<vscale; i++) 
        {   setrow(x, y, w, buffer);
            y += yincr;
        }
     }

     free(buffer);
     return 0; 
} 
