#include <vga.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include "tetconfig.h"
#ifdef JOYSTICK_SUPPORTED
#warning  "If joystick support is not included in your"
#warning  "system, you need to comment out line "
#warning  "'#define JOYSTICK_SUPPORTED' in tetconfig.h,"
#warning  "otherwise this file will compile with errors"
#include <linux/joystick.h>
#endif
#include "kbcodes.h"
#define Esc 0x1B

/*===================================================*/
/*          Joystick and keyboard processing         */
/*---------------------------------------------------*/
/*       04.04.98                          M.G.      */
/*===================================================*/
#ifdef JOYSTICK_SUPPORTED
static int JoyX1=0, JoyX2=0, JoyY1=0, JoyY2=0;
static int JoyButtonLocked;
#endif
static int JoyFd = -1;

/* Get a key from the keyboard if available
   For special keys returns
        LSB : ASCI code or last byte in ANSI sequence
        MSB:  0 - ASCII character
	      1 - arrow key
              2 - Alt key
	      3 - Functional  key (F1, F2, etc) with no CTRL
*/

int  kbdGet(void)
{
   int key;
   int buffer[10];
   int *buf;

   key = vga_getkey();
   if (key != Esc) return key;

   // Key == Esc look for special key
   buf = buffer;
   while((*buf++=vga_getkey()) != 0);

   buf = buffer;
   if ((key=*buf++) != 0x5b) return key ? 0x200 | key : Esc;    // Meta key
   if ((key=*buf++) != 0x5b) return 0x100 | key;    // Arrow key
   return 0x300 | *buf;			   // Functional key

}



/* Init joystick   */
/* Returns
    0 - all right
    4 - error
*/

int joyInit(void)
{
#ifdef JOYSTICK_SUPPORTED
     if (JoyFd >= 0) close(JoyFd);
     JoyFd = open ("/dev/js0", O_RDONLY);
     JoyButtonLocked = 0;
#else
     JoyFd = -1;
#endif
     return JoyFd < 0 ? 4: 0;
}

void joyDeinit(void)
{
     if (JoyFd >= 0) close (JoyFd);
}

int joyGetStatus(int *JoyX, int *JoyY, int *Buttons)
{
#ifdef JOYSTICK_SUPPORTED
      struct JS_DATA_TYPE js;
      int  status, btns, mask;

      if (JoyFd < 0)
      {  joyInit();
         if (JoyFd < 0) return 4;
      }

      status = read (JoyFd, &js, JS_RETURN);

      if (status != JS_RETURN) return 4;

      *JoyX = js.x;
      *JoyY = js.y;

      btns = js.buttons;
      mask = 2;
      while (mask)
      {  // If button pressed, ignore it if locked
         //           or set locked flag if unlocked
         if (btns & mask)
         {  if (JoyButtonLocked & mask) btns &= ~mask;
                         else JoyButtonLocked |= mask;
         }
         // If button not checked - clear locked mask
         else
                JoyButtonLocked &= ~mask;

         mask >>= 1;
      }

      *Buttons = btns;

      return 0;
#else
     return 4;
#endif
}

int joySetParams(void)
{
#ifdef JOYSTICK_SUPPORTED
	int JoyX, JoyY, Btns;

	if (joyGetStatus(&JoyX, &JoyY, &Btns)) return 4;

	JoyX1 = JoyX >> 1;
        JoyY1 = JoyY >> 1;

        if (JoyX1 == 0 && JoyY1 == 0) return 4;

        JoyX2 = JoyX1 * 3;
        JoyY2 = JoyY1 * 3;
        return 0;
#else
	return 4;
#endif
}


/* Get keyboard or jystick command */

int getJoy(void)
{
#ifdef JOYSTICK_SUPPORTED
   int  JoyX, JoyY, Btn;

   if (joyGetStatus(&JoyX, &JoyY, &Btn) == 0)
   {

      switch(Btn & 0x3)
      { case 1:       /* First button only */
           return JOY_BTN1;

        case 2:       /* Second button only: */
             return JOY_BTN2;

  //      case 3:	      /* Both buttons */
  //           return JOY_BTN3;
      }


      /*  No joystick button is pressed */
      if (JoyY > JoyY2)
      { if (JoyX >= JoyX1 && JoyX <= JoyX2)
                     return JOY_DOWN;
      }
      else
      if (JoyY < JoyY1)
      { if (JoyX >= JoyX1 && JoyX <= JoyX2)
                     return JOY_UP;
      }
      else
      if (JoyX > JoyX2) return JOY_RIGHT;
      else
      if (JoyX < JoyX1) return JOY_LEFT;
  }
#endif
  return 0;
}


int getKeyJoy(void)
{  int ret = getJoy();

   if (!ret) ret = kbdGet();

   return ret;
}


static int IndOfInt(int where[], int what)
{  int i, k;

  for (i=0; (k=where[i])!= '\0'; i++)
      if (k==what) return i;

  return -1;
}


int checkKeys(int *keys)
{  int ind;
   int Btn;

#ifdef JOYSTICK_SUPPORTED
  if (joyGetStatus(&ind, &ind, &Btn) == 0)
  {

    if (Btn & 0x1)
    {  if ((ind = IndOfInt(keys, JOY_BTN1)) >= 0)
                         return ind;
    }

    if (Btn & 0x2)
    {  if ((ind = IndOfInt(keys, JOY_BTN2)) >= 0)
                         return ind;
    }

  }
#endif

  while ((Btn =vga_getkey()) != 0)
  {   if (Btn < 0x100) Btn = toupper(Btn);
      if ((ind = IndOfInt(keys, Btn)) >= 0)
               return ind;
  }

  return -1;
}

int IsJoystickSupported(void)
{  return (JoyFd < 0) ? 0 : 1; }
