#include <time.h>
#include <string.h>
#include <values.h>
#include "tet.h"

char  Figures[NFIGS][8] =  { {1,1,1,1,0,0,0,0},    /* 0 - Line */
			     {0,2,2,0,0,2,2,0},    /* 1 - Square */
			     {3,3,3,0,0,0,3,0},    /* 2 - Leg1   */
			     {4,4,4,0,0,4,0,0},    /* 3 - Hammer */
			     {5,5,5,0,5,0,0,0},    /* 4 - Leg2   */
			     {6,6,0,0,0,6,6,0},    /* 5 - Scale1  */
			     {0,7,7,0,7,7,0,0}};   /* 6 - Scale2  */

int  FigTurns[NFIGS] = {2, 1, 4, 4, 4, 2, 2};

int ShadowY;

extern int BoardX, BoardY, BoardH, BoardW;
extern int ScreenHeight, Level;
extern char  Patterns[][FIG_HEIGHT][FGB_WIDTH];
extern BOARD  Board[BRD_HEIGHT][BRD_WIDTH];

extern BOTTOM_MODE BottomMode;
extern WHERE_WE_ARE where;
extern char FillPats[][8];

static char FigImage[(FIG_DIM+2)*FIG_WIDTH];

void embed_figure(BOARD brd[][BRD_WIDTH], FIGURE figure,
			      int x, int y, int wid, int hei)
{ int x1, y1;
  char c;

  for (y1 = 0; y1 < hei ; y1++)
  {    for (x1 = 0; x1< wid; x1++)
       {  c = figure[y1][x1];
	  if (c != 0) brd[y+y1][x+x1] = c;
       }
   }

}


int test_posit( FIGURE figure, int x,  int y, int wid, int hei)
{ int x1, y1, x2, y2;
  char c;


    for (y1 = 0; y1 < hei ; y1++)
    {  y2 = y+y1;
       for (x1 = 0; x1<wid; x1++)
       {  c = figure[y1][x1];
	  if (c == 0) continue;
	  x2 = x+x1;
	  if ( /* y2 < 0 ||*/  y2 >= BRD_HEIGHT
	    ||  x2 < 0 ||  x2 >= BRD_WIDTH
	    ||  Board[y2][x2])   return 0;
       }

    }


   return 1;
}

void show_figure(FIGURE figure, int px, int py, int tox, int toy)
{
    int  x, y, x1, y1, x2, y2, off;
    int  patno;
    char byte;


    for (y=0; y<toy; y++)
    { y2 = y * FIG_HEIGHT;
      for (y1= 0; y1 < FIG_HEIGHT; y1++)
      {  for (x=0; x < tox; x++)
	 { x2 = x * FIG_WIDTH;
	   patno = figure[y][x];
           off =x2;
	   for (x1= 0; x1 < FGB_WIDTH; x1++)
//	      FigImage[x2+x1] = Patterns[patno][y1][x1];
	   {  byte  = Patterns[patno][y1][x1];
              FigImage[off++] = (byte >> 4) & 0xf;
              FigImage[off++] = byte & 0xf;
           }
	 }

	setrow(px, py+y2+y1, tox*FIG_WIDTH, FigImage);
      }
    }
}


static void markcells(char brd[][BRD_WIDTH], FIGURE figure,
				   int x, int y, int wid, int hei, char c)
{ int x1, y1;

     for (y1 = 0; y1 < hei ; y1++)
	for (x1 = 0; x1< wid; x1++)
	     if (figure[y1][x1] != 0)
		  brd[y+y1][x+x1] = c;
}

int calc_cost(FIGURE figure, int x, int y, int wid, int hei, int icost)
{ int x1, y1, f, b= -1;
  int cost = 0, cnt;
  char c1, c2;

  markcells(Board, figure, x,y, wid, hei, 0x80);
  cost = -MAXINT;


  for (y1=0; y1<BRD_HEIGHT; y1++)
  {  cnt = 0;
     c1 = 0;                      /* presumed to be non-empty */

     for (x1=0; x1 < BRD_WIDTH; x1++)
     { c2 = (Board[y1][x1] == 0);
       if (c2)  cnt++;            /* empty cell */

       if ( c2 != c1) cost ++;
       c1 = c2;
     }

     if (b < 0 && cnt < BRD_WIDTH) b = y1;

     cost += cnt * y1 + c2;
     if (b>=0 && cnt * (y1-b) >= BRD_HEIGHT/2) cost += y1;

     if (cost > icost) goto Ex;  /* return cost; */
  }   /* for y1 */

  for (x1=0; x1<BRD_WIDTH; x1++)
  {  c1 = 1;                /* Presume empty */
     f = 0;
     cnt = 0;

     for (y1=0; y1<BRD_HEIGHT; y1++)
     {  c2 = (Board[y1][x1] == 0);
	if (c2)                     /* Empty cell */
	{
	   if (f) cnt++;            /* Empty cell cover by non-empty one */
	}
	else f = 1;                 /* Non-empty cell */

	if (c2 != c1) cost ++;
	c1 = c2;
     }

     cost += cnt * y1 / 2 + c2;

     if (cost > icost) goto Ex;  /*return cost;*/
  }

Ex:
  markcells(Board, figure, x,y, wid, hei, 0);
  return cost;

}

/*------------------------------------------------------*/
static void after_rot(int i1, FIGURE nfig)
{ int i, j, j1;

  if (i1 > 0)
    { for (i=0; i< FIG_DIM-i1; i++)
        memcpy(nfig[i], nfig[i+i1], FIG_DIM*sizeof(BOARD));

      for (i= FIG_DIM-i1; i<FIG_DIM; i++)
        memset(nfig[i], 0, FIG_DIM * sizeof(BOARD));
    }

  for (j=0; j<FIG_DIM; j++)
    for (i=0; i<FIG_DIM; i++)
       if (nfig[i][j] != 0) goto Stp;


  Stp:
  if (j > 0)
    { j1 = j;
      for (j=0; j<FIG_DIM-j1; j++)
        for (i=0; i<FIG_DIM; i++)
          nfig[i][j] = nfig[i][j+j1];

      for (j= FIG_DIM-j1; j<FIG_DIM; j++)
        for (i=0; i<FIG_DIM; i++)
          nfig[i][j] = 0;
    }

}


void rot_left(FIGURE figure, FIGURE nfig)
{ register int i, j;
  register unsigned  k;
#ifdef TRIPACK
  register int k1;
#endif
  int  i1 = -1;

  for (i=0; i< FIG_DIM; i++)
     for (j = 0; j< FIG_DIM; j++)
     {    k = figure[j][FIG_DIM-1-i];

#ifdef  TRIPACK
          k1 = k & 0xf;
          k = (k>>4) & 0x0fff;
          k |= (k1 << 12);
#endif

         if ((nfig[i][j] = k) != 0 && i1 < 0) i1 = i;
     }

  after_rot(i1, nfig);
}


void rot_right(FIGURE figure, FIGURE nfig)
{ register int i, j, k;
#ifdef TRIPACK
  register int k1;
#endif
  int i1 = -1;


  for (i=0; i< FIG_DIM; i++)
    for (j = 0; j< FIG_DIM; j++)
     {    k = figure[FIG_DIM-1-j][i];

#ifdef  TRIPACK
          k1 = (k >> 12) & 0xf;
          k <<= 4;
          k |= k1;
#endif

          if ((nfig[i][j] = k ) != 0 && i1 < 0) i1 = i;
      }

  after_rot(i1, nfig);

}


/*------------------------------------------------------*/
void cop_figure(FIGURE tofig, FIGURE fromfig)
{ int i;

   for (i = 0; i< FIG_DIM; i++)
       memcpy(tofig[i], fromfig[i], FIG_DIM * sizeof(BOARD));
}

/*---------------------------------------------------------*/

void get_figure(FIGURE figure, int type, int rot)
{   int y;
    BOARD *ptr;
    FIGURE figure1;

    ptr = Figures[type];

    for (y=0; y<PATHEIGHT; y++)
    {  memcpy(figure[y], ptr, FIG_DIM * sizeof(BOARD));
       ptr += FIG_DIM;
    }

    for (y=PATHEIGHT; y<FIG_DIM; y++)
       memset(figure[y], 0, FIG_DIM* sizeof(BOARD));

    for (y =0; y<rot; y++)
    {  rot_right(figure, figure1);
       cop_figure(figure, figure1);
    }
}

/*---------------------------------------------------------*/

int fig_width(FIGURE figure)
{ int x, y;

   for (x= FIG_DIM-1; ; x--)
        for(y = 0; y<FIG_DIM; y++)
          if (figure[y][x]) goto Ret;

  Ret:
     return x+1;
}


int fig_height(FIGURE figure)
{ int x, y;

   for(y = FIG_DIM-1;  ;y--)
       for (x=0; x<FIG_DIM; x++)
        if (figure[y][x]) goto Ret;

  Ret:
      return y+1;
}

/*--------------------------------------------------------*/
void show_shadow(FIGURE figure, int pos)
{  char shadow[BRD_WIDTH], s;
   int  wid, off1;
   int  x, y, x1, fromx, tox;
   int col, row, patno;

   if (figure == NULL)
   {  RECT rc;
      rc.top = ShadowY;  rc.bottom = ShadowY + FIG_HEIGHT;
      rc.left = BoardX;  rc.right = BoardX + BoardW;
      DrawSolidRect(&rc, 0);
      return;
   }

   memset(shadow, '\0', BRD_WIDTH);

   for (col=0; col<FIG_DIM; col++)
   { x1 = pos + col;
     if (x1 >= BRD_WIDTH) break;

     s = 0;
     for (row=0; row<FIG_DIM; row++)
     { patno = figure[row][col];
       if (patno)
       { s = patno; break; }
     }

     shadow[x1] = s;

   }

   fromx = pos-1;
   if (fromx < 0)  fromx = 0;
   tox = pos + FIG_DIM +1 ;
   if (tox > BRD_WIDTH) tox = BRD_WIDTH;
//   fromx = 0;
//   tox = BRD_WIDTH;
   x = BoardX + fromx * FIG_WIDTH;
   y = ShadowY;
   wid = (tox - fromx) * FIG_WIDTH;

   memset(FigImage, '\0', sizeof(FigImage));

   for (row=0; row<FIG_HEIGHT; row++)
   {
      off1 = 0;
      for (col=fromx; col<tox; col++)
      {  patno = shadow[col];

         for (x1= 0; x1 < FGB_WIDTH; x1++)
           {  s = Patterns[patno][row][x1];
              FigImage[off1++] = (s >> 4) & 0xf;
              FigImage[off1++] = s & 0xf;
           }
      }

      setrow(x, y++, wid, FigImage);
   }
}

void show_shadow_frame(void)
{   RECT rc;

    rc.top = ShadowY; rc.bottom = ShadowY + FIG_HEIGHT;
    rc.left = BoardX; rc.right = BoardX + BoardW;

    if (BottomMode == B_SHADOW && where == W_GAME)
            DrawFramedRect(&rc);
    else
    {  rc.top -= 3;  rc.bottom += 8;
       if (rc.bottom > ScreenHeight) rc.bottom = ScreenHeight;
       rc.left -=8;  rc.right += 8;
       DrawPatRect(&rc, FillPats[Level], 5-(BoardY%2));
    }
}

