#include <vga.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <ctype.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <pwd.h>        // For getting real name throgh getpwuid
#include <unistd.h>     // For getting user login name
#include "tet.h"
#include "kbcodes.h"
#include "keyjoy.h"
#include "egafont.h"

/* Global data */
int ScreenWidth, ScanWidth, ScreenHeight;
int ScreenColours, VideoChipset;
int VgaVideoMode;
int ScrollAvailable;
int TimerSuspended = 0;
int Intro = 1;
int BeepsEnabled, MusicEnabled;
char *Player = NULL;
BOTTOM_MODE BottomMode;
WHERE_WE_ARE where;
char *PauseFileName = NULL, *PlayListFileName = NULL;
char BgColourName[41];
long   Ticks = 0;
char  FontFileName[41];

extern unsigned int Score;             // tetgame.c
extern int  Demo;                      // tetgame.c
extern int  JoyEnabled;                // tetgame.c
extern int  DefLevel, DefLays;         // tetgame.c
extern int  JoyEquiv[2];               // tetgame.c

static   char   *Font = NULL;

/* Local data */
enum { EGA_MODE, VGA_MODE, SVGA_MODE, LOW_MODE}  VideoMode;

static int ProceedArguments(int argc, char *argv[])
{
   int i;
   char *ptr, c;

   PlayListFileName = GetFullFileName(BGLIST_FILE_NAME);
   PauseFileName  = GetFullFileName(PAUSE_FILE_NAME);
   strcpy(BgColourName, DEFAULT_BG_RGB);
   strcpy(FontFileName, DEFAULT_FONT);

#ifdef SCROLLER_DISABLED
    ScrollAvailable = 0;
#else
    ScrollAvailable = 1;
#endif


#ifdef JOYSTICK_SUPPORTED
   JoyEnabled = 1;
#else
   JoyEnabled = 0;
#endif

   DefLevel  = 0;
   DefLays   = 0;
   VideoMode = DEFAULT_VIDEO;

   JoyEquiv[0] = '5';
   JoyEquiv[1] = '0';

   Demo = 0;
   Intro = 1;

   for (i=1; i<argc; i++)
    {  ptr = argv[i];
       c = *ptr;
       if (c == '/' || c =='-') c = *(++ptr);

       c = toupper(c);

       switch(c)
       {//case 'A':
        //    if ( toupper(*(++ptr)) == 'N')
        //           SoundCard = 0;        /* Do not check for Sound Card */
       //     break;

        case 'I':
            Intro = 0;

       case 'J':
            if (JoyEnabled && (c = *(++ptr)) != 0)
            {  JoyEquiv[0] = toupper(c);
               if ((c = *(++ptr))!=0) JoyEquiv[1] = toupper(c);
            }
            break;

         case 'K':
            JoyEnabled = 0;
            break;

         case 'L':
            c = *(++ptr);
            if (isdigit(c)) DefLevel = c - '0';
            break;

         case 'R':
            c = *(++ptr);
            if (isdigit(c)) DefLays = c - '0';
            break;

         case 'V':
            c = toupper(*(++ptr));
            if (c == 'L') VideoMode = LOW_MODE;
            else
            if (c == 'V') VideoMode = VGA_MODE;
            else
            if (c == 'E') VideoMode = EGA_MODE;
            break;

         case 'D':
            Demo = 1;
            break;

         case 'M':
            if (PlayListFileName) free(PlayListFileName);
            if (strlen(++ptr) == 0) PlayListFileName = NULL;
            else PlayListFileName = GetFullFileName(ptr);
            break;

         case 'P':
            if (PauseFileName) free(PauseFileName);
            if (strlen(++ptr) == 0) PauseFileName = NULL;
            else PauseFileName = GetFullFileName(ptr);
            break;

         case 'S':
            ScrollAvailable = 0;
            break;

         case 'C':
            strcpy(BgColourName, ++ptr);
            break;

         case 'F':
            strcpy(FontFileName, ++ptr);
            break;

         default:
            ShowComHelp();
            return 4;
       }  /* switch(c); */

    } /* for loop */

    switch(VideoMode)
    {  case EGA_MODE:
           VideoChipset = EGA;
           break;

       case LOW_MODE:
           VideoMode = EGA_MODE;
                                 // No break here!
       default:
           VideoChipset = VGA;
    }


    return 0;
}

void IntervalHandler(int code)
{


    if (TimerSuspended)
    {
       if (TimerSuspended == 2) TimerSuspended = 1;
    }
    else
    {
       Ticks++;
 #ifndef SCROLLER_DISABLED
       if (where==W_PROLOGUE || (Demo && BottomMode == B_SCROLLER))
                                                 ScrollerTick();
//     else
 #endif
//       if (BottomMode == B_HELP && where == W_GAME)
//         ChangeHelpPage();
    }
    signal(SIGALRM, IntervalHandler);
//  signal(SIGVTALRM, IntervalHandler);
}

void SuspendTimer(void)
{
   signal(SIGALRM, SIG_IGN);

//   if (TimerSuspended==1) return;
//   TimerSuspended = 2;
//   while (TimerSuspended == 2);
}

void ResumeTimer(void)
//{  TimerSuspended = 0; }
{  signal(SIGALRM, IntervalHandler); }


static void TerminationProc(void)
{
     DeinitScroller();
     joyDeinit();

     egaFreeFont(Font);
     deinitPalette();
     vga_unlockvc();
     vga_unlockvc();
     vga_setmode(TEXT);
     DeinitPlayLines();
     if (PlayListFileName) free(PlayListFileName);

     if (Player) free (Player);

}


void TerminationHandler(int code)
{
    TerminationProc();

    exit(8);
}

int main(int argc, char *argv[])
{  struct itimerval value, oldvalue;
   struct passwd   *p;
   char   *FontPath;
   int (*move_proc)(void);
   char   message[121] = "";
   int    rc;
   int    vgaVideoModes[] = {G640x480x256, G640x480x16, G640x350x16};
   int    m;

   BottomMode = B_NONE;
   rc = ProceedArguments(argc, argv);
   if (rc) return rc;

   rc = 8;

   InitPlayLines();
   MusicEnabled = (PlayListFileName != NULL);

   if (JoyEnabled)
   { if (joyInit() || joySetParams())
                    JoyEnabled = 0;
   }

   FontPath = malloc(257);
   strcpy(FontPath, CONSOLE_FONT_DIR);
   m = strlen(FontPath);
   if (FontPath[m-1] != '/') strcat(FontPath, "/");
   strcat(FontPath, FontFileName);

   Font = egaLoadFont(FontPath);

   if (!Font)
   {  FontPath[80] = '\0';
      sprintf(message, "Font %s is not found or unsupported", FontPath);
   }

   free(FontPath);

   if (!Font) goto Ret;

// Preventing vga_init malfunctioning
   if (VideoMode != SVGA_MODE)
               vga_setchipset(VGA);

   if (vga_init())
   {   strcpy(message, "VGA initialization error");
       goto Ret;
   }

   signal(SIGTERM, TerminationHandler);
   signal(SIGKILL, TerminationHandler);

   if (VideoChipset != EGA)
      VideoChipset = vga_getcurrentchipset();

   m =  (VideoMode == SVGA_MODE) ? 0 :
        (VideoMode == VGA_MODE)  ? 1 : 2;

   while (m<3)
   {  VgaVideoMode = vgaVideoModes[m];
      if (!vga_setmode(VgaVideoMode)) break;
      m++;
   }

   if (m>=3)
   { strcpy(message, "None of possible modes are supported by your card");
     goto Ret;
   }




   ScreenWidth = vga_getxdim();
   ScreenHeight = vga_getydim();
   ScreenColours = vga_getcolors();
   initPalette();
   if (ScreenColours == 16) ScanWidth = ScreenWidth >> 3;
                  else      ScanWidth = ScreenWidth;

   egaSetFont(Font);

   InitBoard();
   InitScroller();

   vga_lockvc();

   // Getting player name
   p =  getpwuid(getuid());

   if (p)                           // Try to get full name
       Player = strdup(p->pw_gecos);
   else                             // If failed, get login name
       Player = strdup(getlogin());


   // Making tick equal to standard PC interval timer tick
   value.it_interval.tv_sec = value.it_value.tv_sec = 0l;
   value.it_interval.tv_usec = value.it_value.tv_usec = 10000000l / 182 ;
//   value.it_interval.tv_usec = value.it_value.tv_usec = 10000000l / 364 ;

   setitimer(ITIMER_REAL, &value, &oldvalue);
   signal(SIGALRM, IntervalHandler);
//   setitimer(ITIMER_VIRTUAL, &value, &oldvalue);
//   signal(SIGVTALRM, IntervalHandler);

   if (Intro)
   {  rc = ShowPrologue();
      if (rc == 8) goto Ret;
      if (rc == 4) Demo = 1;
   }

   InitIntergameScreen();

   if(Demo == 0)
   {
       Score = 0;
       rc = 4;
       if (ShowIntergameScreen()) goto Ret;

   }

   rc = 0;
/* Play game */

    while(rc>=0)
    {  NewGame();

       move_proc = Demo ? move_demo_figure : move_figure;
       while ((rc=(*move_proc)()) == 0 );
       if ( EndGame(rc-4) > 0) break;

     }  /* while(rc); */

     rc = 0;

     signal(SIGALRM, SIG_DFL);
     setitimer(ITIMER_REAL, &oldvalue, &oldvalue);

Ret:
     TerminationProc();

     if (rc > 4 && strlen(message)>0) printf("%s\n", message);
     return rc;
}


